#!/usr/bin/env python

import datetime
import json
import operator
import sys
from collections import defaultdict


nested_dict = lambda: defaultdict(nested_dict)


def split_name(name):
    """ Split a
    """
    return name.split(".")


def read_result_file(result_file):
    with open(result_file, "r") as f:
        return json.load(f)


def parse_result_file(result):
    times = defaultdict(lambda: datetime.timedelta())

    hierarchy = nested_dict()

    total_time = datetime.timedelta()

    for name, start in result["started_at"].items():

        top_level = hierarchy

        seconds = (result["ended_at"][name] - start) / 1000.0
        delta = datetime.timedelta(seconds=seconds)
        total_time += delta

        splitted_name = split_name(name)

        for i, part in enumerate(splitted_name):
            full_part_name = ".".join(splitted_name[: i + 1])

            # Store the hierarchy of results
            top_level = top_level[full_part_name]

            times[full_part_name] += delta

    return times, hierarchy, total_time


def format_results(times, hierarchy, total_time):
    fmt = "{:<70} {:>10} {:>10}%"
    print(fmt.format("Benchmark", "Time", ""))
    print(fmt.format("Total", total_time, "100"))

    format_sub_results(times, hierarchy, total_time)


def format_sub_results(times, hierarchy, total_time, level=0):
    fmt = "{:<70} {:>10} {:>10}%"
    for full_name in sorted(
        hierarchy.keys(), key=lambda bname: times[bname], reverse=True
    ):

        name = full_name.split(".")[-1]

        btime = times[full_name]
        percent = "%.2f" % (btime.total_seconds() / total_time.total_seconds() * 100,)
        fmt_level = " " * (level * 2)
        print(fmt.format(fmt_level + name, btime, percent))

        sub_benchmarks = hierarchy[full_name]

        if sub_benchmarks:
            format_sub_results(times, sub_benchmarks, total_time, level=level + 1)

    # for name, btime in sorted(times.items(), key=operator.itemgetter(1), reverse=True):


def main(result_file):
    result = read_result_file(result_file)

    times, hierarchy, total_time = parse_result_file(result)

    format_results(times, hierarchy, total_time)


if __name__ == "__main__":
    main(sys.argv[1])
