====================================================
Test finding the ASV variants based on format values
====================================================

Setup

  $ export PATH=$TESTDIR/../repo-scripts/:${PATH}

  $ mkdir some-dir
  $ hg init some-dir/my-source-repo --config format.usegeneraldelta=False
  $ hg -R some-dir/my-source-repo debugbuilddag  --new-file '.+5:brancha$.+11:branchb$.+30:branchc<brancha+2<branchb+2'

Create several variant of the same repo
---------------------------------------

  $ ls -1
  some-dir

  $ cat << EOF > no-sparse-revlog-hgrc
  > [format]
  > sparse-revlog = false
  > EOF

  $ cat << EOF > no-general-delta-hgrc
  > [format]
  > usegeneraldelta = false
  > EOF

Define partial set

  $ cat << EOF >> p-config.yaml
  > partial-sets:
  >   same:
  >   missing-last-10:
  >     remove: "last(all(), 10)"
  > EOF

  $ make-reference some-dir/my-source-repo my-reference-I p-config.yaml --hgrc no-sparse-revlog-hgrc
  copy of old repository backed up at $TESTTMP/my-reference-I-tmp?????-reference/.hg/upgradebackup.* (glob)
  the old repository will not be deleted; remove it to free up disk space once the upgraded repository is verified
  hg cloning repository from: some-dir/my-source-repo
  copying hgrc file from: no-sparse-revlog-hgrc
  Running debugupgraderepo because of format mismatch:
  	generaldelta: False != True (repo/config)
  building a reference tarball
  result available at: my-reference-I-d9c2fe76-reference.tar

  $ make-reference some-dir/my-source-repo my-reference-I p-config.yaml --hgrc no-general-delta-hgrc
  hg cloning repository from: some-dir/my-source-repo
  copying hgrc file from: no-general-delta-hgrc
  building a reference tarball
  result available at: my-reference-I-2b380e39-reference.tar

  $ ls -1
  my-reference-I-2b380e39-reference
  my-reference-I-2b380e39-reference.tar
  my-reference-I-2b380e39.benchrepo
  my-reference-I-d9c2fe76-reference
  my-reference-I-d9c2fe76-reference.tar
  my-reference-I-d9c2fe76.benchrepo
  no-general-delta-hgrc
  no-sparse-revlog-hgrc
  p-config.yaml
  some-dir

  $ cat my-reference-I-2b380e39.benchrepo
  reference-repo:
    enabled: true
    format-info:
      compression: zlib
      compression-level: default
      dotencode: true
      fncache: true
      generaldelta: false
      plain-cl-delta: true
      sparserevlog: false
    hg-version: * (glob)
    id: my-reference-I
    number-heads:
      all: 3
      visible: 3
    number-named-branch:
      all: 1
      visible: 1
    number-revisions:
      all: 53
      visible: 53
    partial-sets:
      missing-last-10:
        remove: last(all(), 10)
      same: null
    repo-hash: 2b380e39
    repo-prefix: my-reference-I-2b380e39
    source: some-dir/my-source-repo

  $ cat my-reference-I-d9c2fe76.benchrepo
  reference-repo:
    enabled: true
    format-info:
      compression: zlib
      compression-level: default
      dotencode: true
      fncache: true
      generaldelta: true
      plain-cl-delta: true
      sparserevlog: false
    hg-version: * (glob)
    id: my-reference-I
    number-heads:
      all: 3
      visible: 3
    number-named-branch:
      all: 1
      visible: 1
    number-revisions:
      all: 53
      visible: 53
    partial-sets:
      missing-last-10:
        remove: last(all(), 10)
      same: null
    repo-hash: d9c2fe76
    repo-prefix: my-reference-I-d9c2fe76
    source: some-dir/my-source-repo

Call the function to get the ASV variants
-----------------------------------------

  $ export PYTHONPATH=$TESTDIR/../benchmarks/
  $ python -c "import pprint; import utils; pprint.pprint(utils.get_repo_variants('.'))"
  ([['my-reference-I'],
    ['zlib'],
    ['default'],
    [True],
    [True],
    [False, True],
    [True],
    [False]],
   ['repo',
    'repo-format-compression',
    'repo-format-compression-level',
    'repo-format-dotencode',
    'repo-format-fncache',
    'repo-format-generaldelta',
    'repo-format-plain-cl-delta',
    'repo-format-sparserevlog'],
   8,
   {'my-reference-I-2b380e39': {'reference-repo': {'enabled': True,
                                                   'format-info': {'compression': 'zlib',
                                                                   'compression-level': 'default',
                                                                   'dotencode': True,
                                                                   'fncache': True,
                                                                   'generaldelta': False,
                                                                   'plain-cl-delta': True,
                                                                   'sparserevlog': False},
                                                   'hg-version': '*', (glob)
                                                   'id': 'my-reference-I',
                                                   'number-heads': {'all': 3,
                                                                    'visible': 3},
                                                   'number-named-branch': {'all': 1,
                                                                           'visible': 1},
                                                   'number-revisions': {'all': 53,
                                                                        'visible': 53},
                                                   'partial-sets': {'missing-last-10': {'remove': 'last(all(), 10)'},
                                                                    'same': None},
                                                   'repo-hash': '2b380e39',
                                                   'repo-prefix': 'my-reference-I-2b380e39',
                                                   'source': 'some-dir/my-source-repo'}},
    'my-reference-I-d9c2fe76': {'reference-repo': {'enabled': True,
                                                   'format-info': {'compression': 'zlib',
                                                                   'compression-level': 'default',
                                                                   'dotencode': True,
                                                                   'fncache': True,
                                                                   'generaldelta': True,
                                                                   'plain-cl-delta': True,
                                                                   'sparserevlog': False},
                                                   'hg-version': '*', (glob)
                                                   'id': 'my-reference-I',
                                                   'number-heads': {'all': 3,
                                                                    'visible': 3},
                                                   'number-named-branch': {'all': 1,
                                                                           'visible': 1},
                                                   'number-revisions': {'all': 53,
                                                                        'visible': 53},
                                                   'partial-sets': {'missing-last-10': {'remove': 'last(all(), 10)'},
                                                                    'same': None},
                                                   'repo-hash': 'd9c2fe76',
                                                   'repo-prefix': 'my-reference-I-d9c2fe76',
                                                   'source': 'some-dir/my-source-repo'}}},
   {('my-reference-I', (('repo-format-compression', 'zlib'), ('repo-format-compression-level', 'default'), ('repo-format-dotencode', True), ('repo-format-fncache', True), ('repo-format-generaldelta', False), ('repo-format-plain-cl-delta', True), ('repo-format-sparserevlog', False))): 'my-reference-I-2b380e39',
    ('my-reference-I', (('repo-format-compression', 'zlib'), ('repo-format-compression-level', 'default'), ('repo-format-dotencode', True), ('repo-format-fncache', True), ('repo-format-generaldelta', True), ('repo-format-plain-cl-delta', True), ('repo-format-sparserevlog', False))): 'my-reference-I-d9c2fe76'},
   ['missing-last-10', 'same'])
