==================================
Test for the `make-partial` script
==================================

Setup

  $ export PATH=$TESTDIR/../repo-scripts/:${PATH}

  $ mkdir some-dir
  $ hg init some-dir/source_A
  $ hg -R some-dir/source_A debugbuilddag  --new-file '.+5:brancha$.+11:branchb$.+30:branchc<brancha+2<branchb+2'
  $ hg init some-dir/source_B
  $ hg -R some-dir/source_B debugbuilddag  --new-file '+2:tbase @name1 +3:thead1 <tbase @name2 +4:thead2 @both /thead1 +2:tmaintip'
  $ mkdir repo-references
  $ cd repo-references

Define partial set

  $ cat << EOF >> ../p-config.yaml
  > partial-sets:
  >   same:
  >   missing-last-10:
  >     remove: "last(all(), 10)"
  > EOF
  $ cat << EOF >> ../p-config2.yaml
  > partial-sets:
  >   same:
  >   missing-last-10:
  >     remove: "last(all(), 10)"
  >   missing-last-100:
  >     remove: "last(all(), 100)"
  > EOF

  $ make-reference ../some-dir/source_A repo4test_A ../p-config.yaml
  hg cloning repository from: ../some-dir/source_A
  building a reference tarball
  result available at: repo4test_A-17702431-reference.tar
  $ make-reference ../some-dir/source_B repo4test_B ../p-config2.yaml
  hg cloning repository from: ../some-dir/source_B
  building a reference tarball
  result available at: repo4test_B-67049247-reference.tar
  $ cd ..

Unbundle local reference

  $ mkdir local-references
  $ cd local-references/
  $ tar xf ../repo-references/repo4test_A-*-reference.tar --warning=no-timestamp
  $ tar xf ../repo-references/repo4test_B-*-reference.tar --warning=no-timestamp
  $ cd ..


Test buiding partial repository (one)
-------------------------------------

make a directory for partial repo

  $ mkdir partials-one
  $ cd partials-one

  $ make-partial-repos ../local-references/repo4test_A-*.benchrepo
  Cloning repo4test_A (../local-references/repo4test_A-17702431-reference stripped of last(all(), 10)) into ./repo4test_A-17702431-partial-missing-last-10 (for exchange benchmarks)
  Cloning repo4test_A (../local-references/repo4test_A-17702431-reference stripped of None) into ./repo4test_A-17702431-partial-same (for exchange benchmarks)
  $ ls -1
  repo4test_A-17702431-partial-missing-last-10
  repo4test_A-17702431-partial-missing-last-10.tar
  repo4test_A-17702431-partial-same
  repo4test_A-17702431-partial-same.tar
  $ hg -R ../local-references/repo4test_A-*-reference tip -T '{rev}:{node}\n'
  52:99cae3713489c109d3f91f755bd6d6ea0363d0d5
  $ hg -R repo4test_A-*-partial-missing-last-10 tip -T '{rev}:{node}\n'
  42:511665bca184225b9a5d21d3ffd595045cc2fde7
  $ cat repo4test_A-*-partial-missing-last-10/.hg/ASV_EXPECTED_TIP_REV_HASH
  42 511665bca184225b9a5d21d3ffd595045cc2fde7 (no-eol)
  $ hg -R repo4test_A-*-partial-same tip -T '{rev}:{node}\n'
  52:99cae3713489c109d3f91f755bd6d6ea0363d0d5
  $ cat repo4test_A-*-partial-same/.hg/ASV_EXPECTED_TIP_REV_HASH
  52 99cae3713489c109d3f91f755bd6d6ea0363d0d5 (no-eol)

  $ cd ..

Test buiding partial repositories (multiple)
--------------------------------------------

make a directory for partial repo

  $ mkdir partials-multiple
  $ cd partials-multiple

  $ make-partial-repos ../local-references/*.benchrepo
  Cloning repo4test_A (../local-references/repo4test_A-17702431-reference stripped of last(all(), 10)) into ./repo4test_A-17702431-partial-missing-last-10 (for exchange benchmarks)
  Cloning repo4test_A (../local-references/repo4test_A-17702431-reference stripped of None) into ./repo4test_A-17702431-partial-same (for exchange benchmarks)
  Cloning repo4test_B (../local-references/repo4test_B-67049247-reference stripped of last(all(), 10)) into ./repo4test_B-67049247-partial-missing-last-10 (for exchange benchmarks)
  Cloning repo4test_B (../local-references/repo4test_B-67049247-reference stripped of last(all(), 100)) into ./repo4test_B-67049247-partial-missing-last-100 (for exchange benchmarks)
  Cloning repo4test_B (../local-references/repo4test_B-67049247-reference stripped of None) into ./repo4test_B-67049247-partial-same (for exchange benchmarks)
  $ ls -1
  repo4test_A-17702431-partial-missing-last-10
  repo4test_A-17702431-partial-missing-last-10.tar
  repo4test_A-17702431-partial-same
  repo4test_A-17702431-partial-same.tar
  repo4test_B-67049247-partial-missing-last-10
  repo4test_B-67049247-partial-missing-last-10.tar
  repo4test_B-67049247-partial-missing-last-100
  repo4test_B-67049247-partial-missing-last-100.tar
  repo4test_B-67049247-partial-same
  repo4test_B-67049247-partial-same.tar

  $ tar tf repo4test_B-*-partial-same.tar | sort
  ./repo4test_B-67049247-partial-same/
  ./repo4test_B-67049247-partial-same/.hg/
  ./repo4test_B-67049247-partial-same/.hg/00changelog.i
  ./repo4test_B-67049247-partial-same/.hg/ASV_EXPECTED_TIP_REV_HASH
  ./repo4test_B-67049247-partial-same/.hg/cache/
  ./repo4test_B-67049247-partial-same/.hg/cache/branch2-served
  ./repo4test_B-67049247-partial-same/.hg/cache/rbc-names-v1
  ./repo4test_B-67049247-partial-same/.hg/cache/rbc-revs-v1
  ./repo4test_B-67049247-partial-same/.hg/cache/tags2
  ./repo4test_B-67049247-partial-same/.hg/cache/tags2-served
  ./repo4test_B-67049247-partial-same/.hg/hgrc
  ./repo4test_B-67049247-partial-same/.hg/requires
  ./repo4test_B-67049247-partial-same/.hg/store/
  ./repo4test_B-67049247-partial-same/.hg/store/00changelog.i
  ./repo4test_B-67049247-partial-same/.hg/store/00manifest.i
  ./repo4test_B-67049247-partial-same/.hg/store/data/
  ./repo4test_B-67049247-partial-same/.hg/store/data/nf0.i
  ./repo4test_B-67049247-partial-same/.hg/store/data/nf1.i
  ./repo4test_B-67049247-partial-same/.hg/store/data/nf10.i
  ./repo4test_B-67049247-partial-same/.hg/store/data/nf11.i
  ./repo4test_B-67049247-partial-same/.hg/store/data/nf2.i
  ./repo4test_B-67049247-partial-same/.hg/store/data/nf3.i
  ./repo4test_B-67049247-partial-same/.hg/store/data/nf4.i
  ./repo4test_B-67049247-partial-same/.hg/store/data/nf5.i
  ./repo4test_B-67049247-partial-same/.hg/store/data/nf6.i
  ./repo4test_B-67049247-partial-same/.hg/store/data/nf7.i
  ./repo4test_B-67049247-partial-same/.hg/store/data/nf8.i
  ./repo4test_B-67049247-partial-same/.hg/store/data/nf9.i
  ./repo4test_B-67049247-partial-same/.hg/store/fncache
  $ tar -axf repo4test_B-*-partial-same.tar ./repo4test_B-*-partial-same/.hg/ASV_EXPECTED_TIP_REV_HASH -O
  11 c0808a653b4884c4dcda90aa39bf6dca45972de3 (no-eol)
  $ tar -axf repo4test_B-*-partial-same.tar ./repo4test_B-*-partial-same/.hg/hgrc -O
  # example repository config (see 'hg help config' for more info)
  [paths]
  default = $TESTTMP/some-dir/source_B
  
  # path aliases to other clones of this repo in URLs or filesystem paths
  # (see 'hg help config.paths' for more info)
  #
  # default:pushurl = ssh://jdoe@example.net/hg/jdoes-fork
  # my-fork         = ssh://jdoe@example.net/hg/jdoes-fork
  # my-clone        = /home/jdoe/jdoes-clone
  
  [ui]
  # name and email (local to this repository, optional), e.g.
  # username = Jane Doe <jdoe@example.com>
  $ tar -axf repo4test_B-*-partial-same.tar ./repo4test_B-*-partial-same/.hg/requires -O
  dotencode
  fncache
  generaldelta
  revlogv1
  sparserevlog
  store

  $ cd ..

Test buiding partial repositories some disabled
--------------------------------------------

make a directory for partial repo

  $ mkdir partials-disabled
  $ cd partials-disabled

  $ grep enabled ../local-references/repo4test_A-*.benchrepo
    enabled: true
  $ sed -i 's/enabled: true/enabled: false/' ../local-references/repo4test_A-*.benchrepo
  $ grep enabled ../local-references/repo4test_A-*.benchrepo
    enabled: false
  $ make-partial-repos ../local-references/*.benchrepo
  skipping disabled reference repository: repo4test_A
  Cloning repo4test_B (../local-references/repo4test_B-67049247-reference stripped of last(all(), 10)) into ./repo4test_B-67049247-partial-missing-last-10 (for exchange benchmarks)
  Cloning repo4test_B (../local-references/repo4test_B-67049247-reference stripped of last(all(), 100)) into ./repo4test_B-67049247-partial-missing-last-100 (for exchange benchmarks)
  Cloning repo4test_B (../local-references/repo4test_B-67049247-reference stripped of None) into ./repo4test_B-67049247-partial-same (for exchange benchmarks)
  $ ls -1
  repo4test_B-67049247-partial-missing-last-10
  repo4test_B-67049247-partial-missing-last-10.tar
  repo4test_B-67049247-partial-missing-last-100
  repo4test_B-67049247-partial-missing-last-100.tar
  repo4test_B-67049247-partial-same
  repo4test_B-67049247-partial-same.tar
  $ sed -i 's/enabled: No/enabled: Yes/' ../local-references/repo4test_A-*.benchrepo
  $ grep enabled ../local-references/repo4test_A-*.benchrepo
    enabled: false
  $ cd ..

Test buiding partial from a special format (one)
-------------------------------------

make a reference with special config

  $ cd repo-references
  $ hg clone --quiet --pull --config format.dotencode=no ../some-dir/source_A repo4test_C-existing-reference
  $ cat << EOF > repo4test_C-existing-reference/.hg/hgrc
  > [format]
  > dotencode = no
  > EOF
  $ make-reference ../some-dir/source_A repo4test_C ../p-config.yaml --existing-repo repo4test_C-existing-reference
  using 'repo4test_C-existing-reference' as source instead of cloning
  (make sure this was cloned with the right version)
  building a reference tarball
  result available at: repo4test_C-fbc7bb74-reference.tar
  $ cd ../local-references
  $ tar xf ../repo-references/repo4test_C-*-reference.tar --warning=no-timestamp
  $ cat repo4test_C-*-reference/.hg/hgrc
  [format]
  dotencode = no

(requires should not contains"dotencode")

  $ grep dotencode repo4test_C-*-reference/.hg/requires
  [1]
  $ cat repo4test_C-*-reference/.hg/requires
  fncache
  generaldelta
  revlogv1
  sparserevlog
  store

  $ mkdir -p ../partials-with-format
  $ cd ../partials-with-format

Create reference repository using the same format

  $ make-partial-repos ../local-references/repo4test_C-*.benchrepo
  0 files updated, 0 files merged, 2 files removed, 0 files unresolved
  Cloning repo4test_C (../local-references/repo4test_C-fbc7bb74-reference stripped of last(all(), 10)) into ./repo4test_C-fbc7bb74-partial-missing-last-10 (for exchange benchmarks)
  Cloning repo4test_C (../local-references/repo4test_C-fbc7bb74-reference stripped of None) into ./repo4test_C-fbc7bb74-partial-same (for exchange benchmarks)
  $ ls -1
  repo4test_C-fbc7bb74-partial-missing-last-10
  repo4test_C-fbc7bb74-partial-missing-last-10.tar
  repo4test_C-fbc7bb74-partial-same
  repo4test_C-fbc7bb74-partial-same.tar

  $ tar tf repo4test_C-*-partial-same.tar | sort
  ./repo4test_C-fbc7bb74-partial-same/
  ./repo4test_C-fbc7bb74-partial-same/.hg/
  ./repo4test_C-fbc7bb74-partial-same/.hg/00changelog.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/ASV_EXPECTED_TIP_REV_HASH
  ./repo4test_C-fbc7bb74-partial-same/.hg/branch
  ./repo4test_C-fbc7bb74-partial-same/.hg/cache/
  ./repo4test_C-fbc7bb74-partial-same/.hg/cache/branch2-base
  ./repo4test_C-fbc7bb74-partial-same/.hg/cache/rbc-names-v1
  ./repo4test_C-fbc7bb74-partial-same/.hg/cache/rbc-revs-v1
  ./repo4test_C-fbc7bb74-partial-same/.hg/cache/tags2
  ./repo4test_C-fbc7bb74-partial-same/.hg/cache/tags2-served
  ./repo4test_C-fbc7bb74-partial-same/.hg/dirstate
  ./repo4test_C-fbc7bb74-partial-same/.hg/hgrc
  ./repo4test_C-fbc7bb74-partial-same/.hg/requires
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/00changelog.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/00manifest.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf0.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf1.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf10.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf11.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf12.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf13.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf14.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf15.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf16.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf17.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf18.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf19.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf2.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf20.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf21.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf22.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf23.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf24.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf25.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf26.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf27.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf28.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf29.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf3.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf30.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf31.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf32.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf33.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf34.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf35.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf36.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf37.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf38.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf39.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf4.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf40.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf41.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf42.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf43.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf44.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf45.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf46.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf47.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf48.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf49.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf5.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf50.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf51.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf52.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf6.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf7.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf8.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/data/nf9.i
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/fncache
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/phaseroots
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/undo
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/undo.backupfiles
  ./repo4test_C-fbc7bb74-partial-same/.hg/store/undo.phaseroots
  ./repo4test_C-fbc7bb74-partial-same/.hg/undo.bookmarks
  ./repo4test_C-fbc7bb74-partial-same/.hg/undo.branch
  ./repo4test_C-fbc7bb74-partial-same/.hg/undo.desc
  ./repo4test_C-fbc7bb74-partial-same/.hg/undo.dirstate
  ./repo4test_C-fbc7bb74-partial-same/.hg/wcache/
  ./repo4test_C-fbc7bb74-partial-same/.hg/wcache/checkisexec
  ./repo4test_C-fbc7bb74-partial-same/.hg/wcache/checklink
  ./repo4test_C-fbc7bb74-partial-same/.hg/wcache/checklink-target
  ./repo4test_C-fbc7bb74-partial-same/.hg/wcache/manifestfulltextcache
  ./repo4test_C-fbc7bb74-partial-same/nf10
  ./repo4test_C-fbc7bb74-partial-same/nf11
  ./repo4test_C-fbc7bb74-partial-same/nf12
  ./repo4test_C-fbc7bb74-partial-same/nf13
  ./repo4test_C-fbc7bb74-partial-same/nf14
  ./repo4test_C-fbc7bb74-partial-same/nf15
  ./repo4test_C-fbc7bb74-partial-same/nf16
  ./repo4test_C-fbc7bb74-partial-same/nf17
  ./repo4test_C-fbc7bb74-partial-same/nf51
  ./repo4test_C-fbc7bb74-partial-same/nf52
  ./repo4test_C-fbc7bb74-partial-same/nf6
  ./repo4test_C-fbc7bb74-partial-same/nf7
  ./repo4test_C-fbc7bb74-partial-same/nf8
  ./repo4test_C-fbc7bb74-partial-same/nf9
  $ tar -axf repo4test_C-*-partial-same.tar ./repo4test_C-*-partial-same/.hg/ASV_EXPECTED_TIP_REV_HASH -O
  52 99cae3713489c109d3f91f755bd6d6ea0363d0d5 (no-eol)
  $ tar -axf repo4test_C-*-partial-same.tar ./repo4test_C-*-partial-same/.hg/hgrc -O
  [format]
  dotencode = no

  $ tar -axf repo4test_C-*-partial-same.tar ./repo4test_C-*-partial-same/.hg/requires -O | grep dotencode
  [1]
  $ tar -axf repo4test_C-*-partial-same.tar ./repo4test_C-*-partial-same/.hg/requires -O
  fncache
  generaldelta
  revlogv1
  sparserevlog
  store
  $ tar -axf repo4test_C-*-partial-missing-last-10.tar ./repo4test_C-*-partial-missing-last-10/.hg/ASV_EXPECTED_TIP_REV_HASH -O
  42 511665bca184225b9a5d21d3ffd595045cc2fde7 (no-eol)
  $ tar -axf repo4test_C-*-partial-missing-last-10.tar ./repo4test_C-*-partial-missing-last-10/.hg/hgrc -O
  [format]
  dotencode = no

  $ tar -axf repo4test_C-*-partial-missing-last-10.tar ./repo4test_C-*-partial-missing-last-10/.hg/requires -O | grep dotencode
  [1]
  $ tar -axf repo4test_C-*-partial-missing-last-10.tar ./repo4test_C-*-partial-missing-last-10/.hg/requires -O
  fncache
  generaldelta
  revlogv1
  sparserevlog
  store
  $ cd ..
