======================================
Test for the `update benchrepo` script
======================================

Setup

  $ export PATH=$TESTDIR/../repo-scripts/:${PATH}

Define partial set

  $ cat << EOF >> p-config.yaml
  > partial-sets:
  >   same:
  >   missing-last-10:
  >     remove: "last(all(), 10)"
  > EOF


Create the reference archive
  $ mkdir some-dir
  $ hg init some-dir/my-source-repo
  $ cd some-dir/
  $ hg -R my-source-repo debugbuilddag  --new-file '.+5:brancha$.+11:branchb$.+30:branchc<brancha+2<branchb+2'
  $ cat << EOF > my-source-repo.benchrepo
  > reference-repo:
  >   id: my-source-repo
  >   source: some-dir/my-source-repo
  > EOF
  $ cat my-source-repo.benchrepo
  reference-repo:
    id: my-source-repo
    source: some-dir/my-source-repo

  $ tar cf "my-source-repo.tar" "my-source-repo.benchrepo" "my-source-repo"

Create the partial repos
  $ tar cf "my-source-repo-partial-same.tar" "my-source-repo"

Make a copy of it for a test later
  $ cp my-source-repo-partial-same.tar my-source-repo-partial-same-copy.tar

Wrong call
----------

  $ update-benchrepo --help
  usage: update-benchrepo [-h] reference_archive.tar partial-sets.yaml
  
  Update a benchrepo reference tar with the repo format data
  
  positional arguments:
    reference_archive.tar
    partial-sets.yaml
  
  optional arguments:
    -h, --help            show this help message and exit

  $ update-benchrepo
  usage: update-benchrepo [-h] reference_archive.tar partial-sets.yaml
  update-benchrepo: error: too few arguments
  [2]

Actual call
-----------

  $ update-benchrepo my-source-repo.tar ../p-config.yaml
  Extracting the tar.
  Reading the original benchrepo file.
  Gathering the new benchrepo data.
  Archiving the new benchrepo file.
  Writing the new reference tar as my-source-repo-*-reference.tar (glob)
  Skipping copy of my-source-repo-partial-missing-last-10.tar, not present in reference archive directory
  Copying my-source-repo-partial-same.tar into my-source-repo-39241d24-partial-same.tar
  Fixing directory name inside my-source-repo-39241d24-partial-same.tar
  Removing the temporary directory.

  $ cd ..

  $ ls some-dir/
  my-source-repo
  my-source-repo-39241d24-partial-same.tar
  my-source-repo-39241d24-reference.tar
  my-source-repo-partial-same-copy.tar
  my-source-repo-partial-same.tar
  my-source-repo.benchrepo
  my-source-repo.tar

Check repository name in the '-same' variante

  $ tar tf some-dir/my-source-repo-*-partial-same.tar --wildcards '*/.hg/requires'
  my-source-repo-39241d24-partial-same/.hg/requires

Check the directory and benchrepo is renamed

  $ tar tf some-dir/my-source-repo-*-reference.tar --wildcards '*/.hg/requires' '*.benchrepo' | sort
  my-source-repo-39241d24-reference/.hg/requires
  my-source-repo-39241d24.benchrepo

  $ tar -xf some-dir/my-source-repo-*-reference.tar --wildcards "my-source-repo-*.benchrepo"

  $ cat my-source-repo-*.benchrepo
  reference-repo:
    format-info:
      compression: zlib
      compression-level: default
      dotencode: true
      fncache: true
      generaldelta: true
      plain-cl-delta: true
      sparserevlog: true
    id: my-source-repo
    repo-hash: 39241d24
    repo-prefix: my-source-repo-39241d24
    source: some-dir/my-source-repo

  $ ls some-dir > first-run

New call to check that the script is idempotent and doesn't rename partials with a good name
--------------------------------------------------------------------------------------------

  $ cd some-dir/

  $ update-benchrepo my-source-repo-*-reference.tar ../p-config.yaml
  Extracting the tar.
  Reading the original benchrepo file.
  Gathering the new benchrepo data.
  Archiving the new benchrepo file.
  Writing the new reference tar as my-source-repo-39241d24-reference.tar
  Skipping copy of my-source-repo-39241d24-partial-missing-last-10.tar, not present in reference archive directory
  Skipping copy of my-source-repo-39241d24-partial-same.tar, it already has the right file name
  Directory name inside my-source-repo-39241d24-partial-same.tar is correct
  Removing the temporary directory.

  $ ls . > ../second-run

  $ diff ../first-run ../second-run

Test with an archive correctly named but with the wrong directory name
----------------------------------------------------------------------

  $ cp my-source-repo-partial-same-copy.tar my-source-repo-39241d24-partial-same.tar

  $ tar tf my-source-repo-*-partial-same.tar  --wildcards '*/.hg/requires'
  my-source-repo/.hg/requires

  $ update-benchrepo my-source-repo-*-reference.tar ../p-config.yaml
  Extracting the tar.
  Reading the original benchrepo file.
  Gathering the new benchrepo data.
  Archiving the new benchrepo file.
  Writing the new reference tar as my-source-repo-39241d24-reference.tar
  Skipping copy of my-source-repo-39241d24-partial-missing-last-10.tar, not present in reference archive directory
  Skipping copy of my-source-repo-39241d24-partial-same.tar, it already has the right file name
  Fixing directory name inside my-source-repo-39241d24-partial-same.tar
  Removing the temporary directory.

  $ tar tf my-source-repo-*-partial-same.tar  --wildcards '*/.hg/requires'
  my-source-repo-39241d24-partial-same/.hg/requires
