=========================================
Test for the `make-validate-repos` script
=========================================

Setup

  $ export PATH=$TESTDIR/../repo-scripts/:${PATH}

  $ mkdir some-dir
  $ hg init some-dir/source_A
  $ hg -R some-dir/source_A debugbuilddag  --new-file '.+5:brancha$.+11:branchb$.+30:branchc<brancha+2<branchb+2'
  $ hg init some-dir/source_B
  $ hg -R some-dir/source_B debugbuilddag  --new-file '+2:tbase @name1 +3:thead1 <tbase @name2 +4:thead2 @both /thead1 +2:tmaintip'
  $ mkdir repo-references
  $ cd repo-references

Define partial set

  $ cat << EOF >> p-config.yaml
  > partial-sets:
  >   same:
  >   missing-last-10:
  >     remove: "last(all(), 10)"
  > EOF

  $ make-reference ../some-dir/source_A repo4test_A p-config.yaml
  hg cloning repository from: ../some-dir/source_A
  building a reference tarball
  result available at: repo4test_A-17702431-reference.tar
  $ make-reference ../some-dir/source_B repo4test_B p-config.yaml
  hg cloning repository from: ../some-dir/source_B
  building a reference tarball
  result available at: repo4test_B-67049247-reference.tar
  $ cd ..

Unbundle local reference

  $ mkdir local-references
  $ cd local-references/
  $ tar xf ../repo-references/repo4test_A-*-reference.tar --warning=no-timestamp
  $ tar xf ../repo-references/repo4test_B-*-reference.tar --warning=no-timestamp
  $ cd ..

Define partial set

  $ cat << EOF >> p-config.yaml
  > partial-sets:
  >   same:
  >   missing-last-10:
  >     remove: "last(all(), 10)"
  > EOF

Test buiding partial repository (one)
-------------------------------------

make a directory for partial repo

  $ mkdir partials-one
  $ cd partials-one

  $ make-partial-repos ../local-references/repo4test_A-*.benchrepo
  Cloning repo4test_A (../local-references/repo4test_A-17702431-reference stripped of last(all(), 10)) into ./repo4test_A-17702431-partial-missing-last-10 (for exchange benchmarks)
  Cloning repo4test_A (../local-references/repo4test_A-17702431-reference stripped of None) into ./repo4test_A-17702431-partial-same (for exchange benchmarks)
  $ ls -1
  repo4test_A-17702431-partial-missing-last-10
  repo4test_A-17702431-partial-missing-last-10.tar
  repo4test_A-17702431-partial-same
  repo4test_A-17702431-partial-same.tar
  $ hg -R ../local-references/repo4test_A-*-reference tip -T '{rev}:{node}\n'
  52:99cae3713489c109d3f91f755bd6d6ea0363d0d5
  $ hg -R repo4test_A-*-partial-missing-last-10 tip -T '{rev}:{node}\n'
  42:511665bca184225b9a5d21d3ffd595045cc2fde7
  $ cat repo4test_A-*-partial-missing-last-10/.hg/ASV_EXPECTED_TIP_REV_HASH
  42 511665bca184225b9a5d21d3ffd595045cc2fde7 (no-eol)
  $ hg -R repo4test_A-*-partial-same tip -T '{rev}:{node}\n'
  52:99cae3713489c109d3f91f755bd6d6ea0363d0d5
  $ cat repo4test_A-*-partial-same/.hg/ASV_EXPECTED_TIP_REV_HASH
  52 99cae3713489c109d3f91f755bd6d6ea0363d0d5 (no-eol)

  $ cd ..

Test checking partial repositories
----------------------------------

  $ cd partials-one
  $ validate-repos .
  ./repo4test_A-17702431-partial-missing-last-10:
      fncache: ok
      tip: ok
  
  ./repo4test_A-17702431-partial-same:
      fncache: ok
      tip: ok
  

Test checking partial repositories empty fncache
------------------------------------------------

wildcard + echo seems not portable enough
  $ cd ./repo4test_A-*-partial-missing-last-10/.hg/store
  $ rm fncache
  $ touch fncache
  $ cd ../../..

  $ wc -c ./repo4test_A-*-partial-missing-last-10/.hg/store/fncache
  0 ./repo4test_A-17702431-partial-missing-last-10/.hg/store/fncache

  $ validate-repos .
  ./repo4test_A-17702431-partial-missing-last-10:
      fncache: KO
      tip: ok
  
  ./repo4test_A-17702431-partial-same:
      fncache: ok
      tip: ok
  
  [1]

  $ rm ./repo4test_A-*-partial-missing-last-10/.hg/store/fncache
  $ validate-repos .
  ./repo4test_A-17702431-partial-missing-last-10:
      fncache: KO
      tip: ok
  
  ./repo4test_A-17702431-partial-same:
      fncache: ok
      tip: ok
  
  [1]

Test checking partial repositories bad tip
------------------------------------------

  $ hg -R repo4test_A-*-partial-same branch "my-branch"
  marked working directory as branch my-branch
  (branches are permanent and global, did you want a bookmark?)
  $ hg -R repo4test_A-*-partial-same commit -m "New branch"

  $ hg -R repo4test_A-*-partial-same tip -T '{rev}:{node}\n'
  53:3bee1aa8c6ea80203fb80e582e9f75f7ab9e6cfa

  $ validate-repos .
  ./repo4test_A-17702431-partial-missing-last-10:
      fncache: KO
      tip: ok
  
  ./repo4test_A-17702431-partial-same:
      fncache: ok
      tip: KO
  
  [1]
